@@ -45,7 +45,6 @@
 #include "Peer.h"
 #include "DHTTokenTracker.h"
 #include "util.h"
-#include "bencode.h"
 
 namespace aria2 {
 
@@ -59,62 +58,54 @@ DHTGetPeersMessage::DHTGetPeersMessage(const SharedHandle<DHTNode>& localNode,
                                        const std::string& transactionID):
   DHTQueryMessage(localNode, remoteNode, transactionID)
 {
-  memcpy(_infoHash, infoHash, DHT_ID_LENGTH);
+  memcpy(infoHash_, infoHash, DHT_ID_LENGTH);
 }
 
 DHTGetPeersMessage::~DHTGetPeersMessage() {}
 
 void DHTGetPeersMessage::doReceivedAction()
 {
-  std::string token = _tokenTracker->generateToken(_infoHash,
-                                                   _remoteNode->getIPAddress(),
-                                                   _remoteNode->getPort());
+  std::string token = tokenTracker_->generateToken
+    (infoHash_, getRemoteNode()->getIPAddress(), getRemoteNode()->getPort());
   // Check to see localhost has the contents which has same infohash
   std::vector<SharedHandle<Peer> > peers;
-  _peerAnnounceStorage->getPeers(peers, _infoHash);
-  SharedHandle<DHTMessage> reply;
-  if(peers.empty()) {
-    std::vector<SharedHandle<DHTNode> > nodes;
-    _routingTable->getClosestKNodes(nodes, _infoHash);
-    reply =
-      _factory->createGetPeersReplyMessage(_remoteNode, nodes, token,
-                                           _transactionID);
-  } else {
-    reply =
-      _factory->createGetPeersReplyMessage(_remoteNode, peers, token,
-                                           _transactionID);
-  }
-  _dispatcher->addMessageToQueue(reply);
+  peerAnnounceStorage_->getPeers(peers, infoHash_);
+  std::vector<SharedHandle<DHTNode> > nodes;
+  getRoutingTable()->getClosestKNodes(nodes, infoHash_);
+  SharedHandle<DHTMessage> reply =
+    getMessageFactory()->createGetPeersReplyMessage
+    (getRemoteNode(), nodes, peers, token, getTransactionID());
+  getMessageDispatcher()->addMessageToQueue(reply);
 }
 
-BDE DHTGetPeersMessage::getArgument()
+SharedHandle<Dict> DHTGetPeersMessage::getArgument()
 {
-  BDE aDict = BDE::dict();
-  aDict[DHTMessage::ID] = BDE(_localNode->getID(), DHT_ID_LENGTH);
-  aDict[INFO_HASH] = BDE(_infoHash, DHT_ID_LENGTH);
+  SharedHandle<Dict> aDict = Dict::g();
+  aDict->put(DHTMessage::ID, String::g(getLocalNode()->getID(), DHT_ID_LENGTH));
+  aDict->put(INFO_HASH, String::g(infoHash_, DHT_ID_LENGTH));
   return aDict;
 }
 
-std::string DHTGetPeersMessage::getMessageType() const
+const std::string& DHTGetPeersMessage::getMessageType() const
 {
   return GET_PEERS;
 }
 
-void DHTGetPeersMessage::validate() const {}
-
-void DHTGetPeersMessage::setPeerAnnounceStorage(const WeakHandle<DHTPeerAnnounceStorage>& storage)
+void DHTGetPeersMessage::setPeerAnnounceStorage
+(const WeakHandle<DHTPeerAnnounceStorage>& storage)
 {
-  _peerAnnounceStorage = storage;
+  peerAnnounceStorage_ = storage;
 }
 
-void DHTGetPeersMessage::setTokenTracker(const WeakHandle<DHTTokenTracker>& tokenTracker)
+void DHTGetPeersMessage::setTokenTracker
+(const WeakHandle<DHTTokenTracker>& tokenTracker)
 {
-  _tokenTracker = tokenTracker;
+  tokenTracker_ = tokenTracker;
 }
 
 std::string DHTGetPeersMessage::toStringOptional() const
 {
-  return "info_hash="+util::toHex(_infoHash, INFO_HASH_LENGTH);
+  return "info_hash="+util::toHex(infoHash_, INFO_HASH_LENGTH);
 }
 
 } // namespace aria2
