@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -37,7 +37,7 @@
 
 #include "common.h"
 #include <string>
-#include <deque>
+#include <vector>
 
 namespace aria2 {
 
@@ -46,19 +46,19 @@ class DownloadHandlerConstants
 public:
   static const char* METALINK_EXTENSIONS[];
 
-  static const std::deque<std::string>& getMetalinkExtensions();
+  static const std::vector<std::string>& getMetalinkExtensions();
 
   static const char* METALINK_CONTENT_TYPES[];
 
-  static const std::deque<std::string>& getMetalinkContentTypes();
+  static const std::vector<std::string>& getMetalinkContentTypes();
 
   static const char* BT_EXTENSIONS[];
 
-  static const std::deque<std::string>& getBtExtensions();
+  static const std::vector<std::string>& getBtExtensions();
 
   static const char* BT_CONTENT_TYPES[];
 
-  static const std::deque<std::string>& getBtContentTypes();
+  static const std::vector<std::string>& getBtContentTypes();
 };
 
 } // namespace aria2
