@@ -42,6 +42,7 @@
 #include "LogFactory.h"
 #include "wallclock.h"
 #include "a2algo.h"
+#include "uri.h"
 
 namespace aria2 {
 
@@ -103,19 +104,18 @@ void FileEntry::getUris(std::vector<std::string>& uris) const
   uris.insert(uris.end(), uris_.begin(), uris_.end());
 }
 
-template<typename InputIterator>
-static size_t countInFlightHost(InputIterator first, InputIterator last,
-                                const std::string& hostname)
+template<typename InputIterator, typename OutputIterator>
+static OutputIterator
+enumerateInFlightHosts
+(InputIterator first, InputIterator last, OutputIterator out)
 {
-  // TODO redirection should be considered here. We need to parse
-  // original URI to get hostname.
-  size_t count = 0;
   for(; first != last; ++first) {
-    if((*first)->getHost() == hostname) {
-      ++count;
+    uri::UriStruct us;
+    if(uri::parse(us, (*first)->getUri())) {
+      *out++ = us.host;
     }
   }
-  return count;
+  return out;
 }
 
 SharedHandle<Request>
@@ -127,75 +127,51 @@ FileEntry::getRequest
  const std::string& method)
 {
   SharedHandle<Request> req;
-  Request r;
-  if(!requestPool_.empty()) {
-    for(std::deque<SharedHandle<Request> >::iterator i = requestPool_.begin(),
-          eoi = requestPool_.end(); i != eoi; ++i) {
-      r.setUri((*i)->getUri());
-      if(findSecond(usedHosts.begin(), usedHosts.end(), r.getHost()) !=
-         usedHosts.end()) {
-        continue;
-      }
-      if(countInFlightHost(inFlightRequests_.begin(), inFlightRequests_.end(),
-                           r.getHost()) >= maxConnectionPerServer_) {
-        continue;
-      }
-      req = *i;
-      requestPool_.erase(i);
-      inFlightRequests_.push_back(req);
-      return req;
-    }
-  }
-
-  for(int g = 0; g < 2; ++g) {
-    std::vector<std::string> pending;
-    std::vector<std::string> ignoreHost;
-    while(1) {
-      std::string uri = selector->select(this, usedHosts);
-      if(uri.empty()) {
-        break;
-      }
-      req.reset(new Request());
-      if(req->setUri(uri)) {
-        if(countInFlightHost(inFlightRequests_.begin(),
-                             inFlightRequests_.end(),
-                             req->getHost()) >= maxConnectionPerServer_) {
-          pending.push_back(uri);
-          ignoreHost.push_back(req->getHost());
-          req.reset();
-          continue;
+  if(requestPool_.empty()) {
+    std::vector<std::string> inFlightHosts;
+    enumerateInFlightHosts(inFlightRequests_.begin(), inFlightRequests_.end(),
+                           std::back_inserter(inFlightHosts));
+    for(int g = 0; g < 2; ++g) {
+      std::vector<std::string> pending;
+      std::vector<std::string> ignoreHost;
+      while(1) {
+        std::string uri = selector->select(this, usedHosts);
+        if(uri.empty()) {
+          break;
         }
-        req->setReferer(referer);
-        req->setMethod(method);
-        spentUris_.push_back(uri);
-        inFlightRequests_.push_back(req);
-        break;
+        req.reset(new Request());
+        if(req->setUri(uri)) {
+          if(std::count(inFlightHosts.begin(),
+                        inFlightHosts.end(),req->getHost())
+             >= static_cast<int>(maxConnectionPerServer_)) {
+            pending.push_back(uri);
+            ignoreHost.push_back(req->getHost());
+            req.reset();
+            continue;
+          }
+          req->setReferer(referer);
+          req->setMethod(method);
+          spentUris_.push_back(uri);
+          inFlightRequests_.push_back(req);
+          break;
+        } else {
+          req.reset();
+        }
+      }
+      uris_.insert(uris_.begin(), pending.begin(), pending.end());
+      if(g == 0 && uriReuse && req.isNull() && uris_.size() == pending.size()) {
+        // Reuse URIs other than ones in pending
+        reuseUri(ignoreHost);
       } else {
-        req.reset();
+        break;
       }
     }
-    uris_.insert(uris_.begin(), pending.begin(), pending.end());
-    // TODO UriReuse is performed only when PREF_REUSE_URI is true.
-    if(g == 0 && uriReuse && req.isNull() && uris_.size() == pending.size()) {
-      // Reuse URIs other than ones in pending
-      reuseUri(ignoreHost);
-    } else {
-      break;
-    }
-  }
-  if(req.isNull()) {
-    Request r;
-    for(std::deque<SharedHandle<Request> >::iterator i = requestPool_.begin(),
-          eoi = requestPool_.end(); i != eoi; ++i) {
-      r.setUri((*i)->getUri());
-      if(countInFlightHost(inFlightRequests_.begin(), inFlightRequests_.end(),
-                           r.getHost()) >= maxConnectionPerServer_) {
-        continue;
-      }
-      req = *i;
-      requestPool_.erase(i);
-      inFlightRequests_.push_back(req);
-      return req;
+  } else {    
+    req = requestPool_.front();
+    requestPool_.pop_front();
+    inFlightRequests_.push_back(req);
+    if(logger_->debug()) {
+      logger_->debug("Picked up from pool: %s", req->getUri().c_str());
     }
   }
   return req;
@@ -229,6 +205,7 @@ FileEntry::findFasterRequest(const SharedHandle<Request>& base)
   return SharedHandle<Request>();
 }
 
+namespace {
 class RequestFaster {
 public:
   bool operator()(const SharedHandle<Request>& lhs,
@@ -244,6 +221,7 @@ public:
       lhs->getPeerStat()->getAvgDownloadSpeed() > rhs->getPeerStat()->getAvgDownloadSpeed();
   }
 };
+}
 
 void FileEntry::storePool(const SharedHandle<Request>& request)
 {
@@ -283,11 +261,13 @@ bool FileEntry::removeRequest(const SharedHandle<Request>& request)
 void FileEntry::removeURIWhoseHostnameIs(const std::string& hostname)
 {
   std::deque<std::string> newURIs;
-  Request req;
   for(std::deque<std::string>::const_iterator itr = uris_.begin(),
         eoi = uris_.end(); itr != eoi; ++itr) {
-    if(((*itr).find(hostname) == std::string::npos) ||
-       (req.setUri(*itr) && (req.getHost() != hostname))) {
+    uri::UriStruct us;
+    if(!uri::parse(us, *itr)) {
+      continue;
+    }
+    if(us.host != hostname) {
       newURIs.push_back(*itr);
     }
   }
@@ -295,7 +275,7 @@ void FileEntry::removeURIWhoseHostnameIs(const std::string& hostname)
     logger_->debug("Removed %d duplicate hostname URIs for path=%s",
                    uris_.size()-newURIs.size(), getPath().c_str());
   }
-  uris_ = newURIs;
+  uris_.swap(newURIs);
 }
 
 void FileEntry::removeIdenticalURI(const std::string& uri)
@@ -308,6 +288,7 @@ void FileEntry::addURIResult(std::string uri, downloadresultcode::RESULT result)
   uriResults_.push_back(URIResult(uri, result));
 }
 
+namespace {
 class FindURIResultByResult {
 private:
   downloadresultcode::RESULT r_;
@@ -319,6 +300,7 @@ public:
     return uriResult.getResult() == r_;
   }
 };
+}
 
 void FileEntry::extractURIResult
 (std::deque<URIResult>& res, downloadresultcode::RESULT r)
@@ -359,11 +341,11 @@ void FileEntry::reuseUri(const std::vector<std::string>& ignore)
                       errorUris.begin(), errorUris.end(),
                       std::back_inserter(reusableURIs));
   std::vector<std::string>::iterator insertionPoint = reusableURIs.begin();
-  Request req;
   for(std::vector<std::string>::iterator i = reusableURIs.begin(),
         eoi = reusableURIs.end(); i != eoi; ++i) {
-    req.setUri(*i);
-    if(std::find(ignore.begin(), ignore.end(), req.getHost()) == ignore.end()) {
+    uri::UriStruct us;
+    if(uri::parse(us, *i) &&
+       std::find(ignore.begin(), ignore.end(), us.host) == ignore.end()) {
       if(i != insertionPoint) {
         *insertionPoint = *i;
       }
@@ -451,7 +433,8 @@ size_t FileEntry::setUris(const std::vector<std::string>& uris)
 
 bool FileEntry::addUri(const std::string& uri)
 {
-  if(Request().setUri(uri)) {
+  uri::UriStruct us;
+  if(uri::parse(us, uri)) {
     uris_.push_back(uri);
     return true;
   } else {
@@ -461,7 +444,8 @@ bool FileEntry::addUri(const std::string& uri)
 
 bool FileEntry::insertUri(const std::string& uri, size_t pos)
 {
-  if(Request().setUri(uri)) {
+  uri::UriStruct us;
+  if(uri::parse(us, uri)) {
     pos = std::min(pos, uris_.size());
     uris_.insert(uris_.begin()+pos, uri);
     return true;
