@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -38,13 +38,18 @@
 #include "RequestGroup.h"
 #include "RecoverableException.h"
 #include "message.h"
+#include "Logger.h"
+#include "DownloadContext.h"
+#include "ServerStatMan.h"
+#include "FileAllocationEntry.h"
+#include "CheckIntegrityEntry.h"
 
-FillRequestGroupCommand::FillRequestGroupCommand(int32_t cuid,
-						 DownloadEngine* e,
-						 int32_t interval):
+namespace aria2 {
+
+FillRequestGroupCommand::FillRequestGroupCommand(cuid_t cuid,
+                                                 DownloadEngine* e):
   Command(cuid),
-  _e(e),
-  _interval(interval)
+  e_(e)
 {
   setStatusRealtime();
 }
@@ -53,15 +58,31 @@ FillRequestGroupCommand::~FillRequestGroupCommand() {}
 
 bool FillRequestGroupCommand::execute()
 {
-  try {
-    _e->_requestGroupMan->fillRequestGroupFromReserver(_e);
-  } catch(RecoverableException* ex) {
-    logger->error(EX_EXCEPTION_CAUGHT, ex);
-    delete ex;
-  }
-  if(_e->_requestGroupMan->downloadFinished() || _e->isHaltRequested()) {
+  if(e_->isHaltRequested()) {
     return true;
   }
-  _e->commands.push_back(this);
+  SharedHandle<RequestGroupMan> rgman = e_->getRequestGroupMan();
+  if(rgman->queueCheckRequested()) {
+    while(rgman->queueCheckRequested()) {
+      try {
+        // During adding RequestGroup,
+        // RequestGroupMan::requestQueueCheck() might be called, so
+        // first clear it here.
+        rgman->clearQueueCheck();
+        rgman->fillRequestGroupFromReserver(e_);
+      } catch(RecoverableException& ex) {
+        getLogger()->error(EX_EXCEPTION_CAUGHT, ex);
+        // Re-request queue check to fulfill the requests of all
+        // downloads, some might come after this exception.
+        rgman->requestQueueCheck();
+      }
+    }
+    if(rgman->downloadFinished()) {
+      return true;
+    }
+  }
+  e_->addRoutineCommand(this);
   return false;
 }
+
+} // namespace aria2
