@@ -141,7 +141,8 @@ bool FtpNegotiationCommand::executeInternal() {
 
 bool FtpNegotiationCommand::recvGreeting() {
   if(!checkIfConnectionEstablished
-     (getSocket(), connectedHostname_, connectedAddr_, connectedPort_)) {
+     (getSocket(), getRequest()->getConnectedHostname(),
+      getRequest()->getConnectedAddr(), getRequest()->getConnectedPort())) {
     sequence_ = SEQ_EXIT;
     return false;
   }
@@ -402,11 +403,17 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
       return false;
     }
 
-    if(getRequestGroup()->downloadFinishedByFileLength()) {
+    if(getDownloadContext()->knowsTotalLength() &&
+       getRequestGroup()->downloadFinishedByFileLength()) {
+      // TODO If metalink file does not contain size and it contains
+      // hash and file is not zero length, but remote server says the
+      // file size is 0, no hash check is performed in the current
+      // implementation. See also
+      // HttpResponseCommand::handleOtherEncoding()
       getRequestGroup()->initPieceStorage();
       getPieceStorage()->markAllPiecesDone();
+      getDownloadContext()->setChecksumVerified(true);
       sequence_ = SEQ_DOWNLOAD_ALREADY_COMPLETED;
-
       getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
                           util::itos(getRequestGroup()->getGID()).c_str(),
                           getRequestGroup()->getFirstFilePath().c_str());
@@ -422,6 +429,7 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
 
     if(getDownloadContext()->knowsTotalLength()) {
       sequence_ = SEQ_DOWNLOAD_ALREADY_COMPLETED;
+      getPieceStorage()->markAllPiecesDone();
       poolConnection();
       return false;
     }
@@ -444,31 +452,20 @@ bool FtpNegotiationCommand::onFileSizeDetermined(uint64_t totalLength)
       return false;
     }
 
-    BtProgressInfoFileHandle infoFile
-      (new DefaultBtProgressInfoFile(getDownloadContext(),
-                                     getPieceStorage(),
-                                     getOption().get()));
-    if(!infoFile->exists() &&
-       getRequestGroup()->downloadFinishedByFileLength()) {
-      getPieceStorage()->markAllPiecesDone();
-
+    SharedHandle<CheckIntegrityEntry> checkIntegrityEntry =
+      getRequestGroup()->createCheckIntegrityEntry();
+    if(checkIntegrityEntry.isNull()) {
       sequence_ = SEQ_DOWNLOAD_ALREADY_COMPLETED;
-      
-      getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                          util::itos(getRequestGroup()->getGID()).c_str(),
-                          getRequestGroup()->getFirstFilePath().c_str());
-
       poolConnection();
-      
       return false;
     }
-    getRequestGroup()->loadAndOpenFile(infoFile);
+    checkIntegrityEntry->pushNextCommand(this);
     // We have to make sure that command that has Request object must
     // have segment after PieceStorage is initialized. See
     // AbstractCommand::execute()
     getSegmentMan()->getSegmentWithIndex(getCuid(), 0);
 
-    prepareForNextAction(this);
+    prepareForNextAction(checkIntegrityEntry);
 
     disableReadCheckSocket();
   }
@@ -951,6 +948,7 @@ void FtpNegotiationCommand::poolConnection() const
 void FtpNegotiationCommand::onDryRunFileFound()
 {
   getPieceStorage()->markAllPiecesDone();
+  getDownloadContext()->setChecksumVerified(true);
   poolConnection();
   sequence_ = SEQ_HEAD_OK;
 }
