@@ -131,7 +131,8 @@ bool HttpRequestCommand::executeInternal() {
   }
   if(httpConnection_->sendBufferIsEmpty()) {
     if(!checkIfConnectionEstablished
-       (getSocket(), connectedHostname_, connectedAddr_, connectedPort_)) {
+       (getSocket(), getRequest()->getConnectedHostname(),
+        getRequest()->getConnectedAddr(), getRequest()->getConnectedPort())) {
       return true;
     }
 
