@@ -46,19 +46,18 @@
 #include "DlRetryEx.h"
 #include "StringFormat.h"
 #include "A2STR.h"
-#include "Decoder.h"
-#include "ChunkedDecoder.h"
-#ifdef HAVE_LIBZ
-# include "GZipDecoder.h"
-#endif // HAVE_LIBZ
 #include "CookieStorage.h"
 #include "AuthConfigFactory.h"
 #include "AuthConfig.h"
+#include "ChunkedDecodingStreamFilter.h"
+#ifdef HAVE_LIBZ
+# include "GZipDecodingStreamFilter.h"
+#endif // HAVE_LIBZ
 
 namespace aria2 {
 
-HttpResponse::HttpResponse():cuid(0),
-                             logger(LogFactory::getInstance())
+HttpResponse::HttpResponse():cuid_(0),
+                             logger_(LogFactory::getInstance())
 {}
 
 HttpResponse::~HttpResponse() {}
@@ -69,27 +68,41 @@ void HttpResponse::validateResponse() const
   if(status >= HttpHeader::S400) {
     return;
   }
-  if(status >= HttpHeader::S300) {
-    if(!httpHeader->defined(HttpHeader::LOCATION)) {
+  if(status == HttpHeader::S304) {
+    if(httpRequest_->getIfModifiedSinceHeader().empty()) {
+      throw DL_ABORT_EX("Got 304 without If-Modified-Since");
+    }
+  } else if(status == HttpHeader::S301 ||
+     status == HttpHeader::S302 ||
+     status == HttpHeader::S303 ||
+     status == HttpHeader::S307) {
+    if(!httpHeader_->defined(HttpHeader::LOCATION)) {
       throw DL_ABORT_EX
         (StringFormat(EX_LOCATION_HEADER_REQUIRED,
                       util::parseUInt(status)).str());
     }
-  } else if(!httpHeader->defined(HttpHeader::TRANSFER_ENCODING)) {
-    // compare the received range against the requested range
-    RangeHandle responseRange = httpHeader->getRange();
-    if(!httpRequest->isRangeSatisfied(responseRange)) {
-      throw DL_ABORT_EX2
-        (StringFormat
-         (EX_INVALID_RANGE_HEADER,
-          util::itos(httpRequest->getStartByte(), true).c_str(),
-          util::itos(httpRequest->getEndByte(), true).c_str(),
-          util::uitos(httpRequest->getEntityLength(), true).c_str(),
-          util::itos(responseRange->getStartByte(), true).c_str(),
-          util::itos(responseRange->getEndByte(), true).c_str(),
-          util::uitos(responseRange->getEntityLength(), true).c_str()).str(),
-         downloadresultcode::CANNOT_RESUME);
+    return;
+  } else if(status == HttpHeader::S200 ||
+            status == HttpHeader::S206) {
+    if(!httpHeader_->defined(HttpHeader::TRANSFER_ENCODING)) {
+      // compare the received range against the requested range
+      RangeHandle responseRange = httpHeader_->getRange();
+      if(!httpRequest_->isRangeSatisfied(responseRange)) {
+        throw DL_ABORT_EX2
+          (StringFormat
+           (EX_INVALID_RANGE_HEADER,
+            util::itos(httpRequest_->getStartByte(), true).c_str(),
+            util::itos(httpRequest_->getEndByte(), true).c_str(),
+            util::uitos(httpRequest_->getEntityLength(), true).c_str(),
+            util::itos(responseRange->getStartByte(), true).c_str(),
+            util::itos(responseRange->getEndByte(), true).c_str(),
+            util::uitos(responseRange->getEntityLength(), true).c_str()).str(),
+           downloadresultcode::CANNOT_RESUME);
+      }
     }
+  } else {
+    throw DL_ABORT_EX
+      (StringFormat("Unexpected status %s", status.c_str()).str());
   }
 }
 
@@ -97,18 +110,18 @@ std::string HttpResponse::determinFilename() const
 {
   std::string contentDisposition =
     util::getContentDispositionFilename
-    (httpHeader->getFirst(HttpHeader::CONTENT_DISPOSITION));
+    (httpHeader_->getFirst(HttpHeader::CONTENT_DISPOSITION));
   if(contentDisposition.empty()) {
-    std::string file = util::percentDecode(httpRequest->getFile());
+    std::string file = util::percentDecode(httpRequest_->getFile());
     if(file.empty()) {
       return "index.html";
     } else {
       return file;
     }
   } else {
-    if(logger->info()) {
-      logger->info(MSG_CONTENT_DISPOSITION_DETECTED,
-                   util::itos(cuid).c_str(), contentDisposition.c_str());
+    if(logger_->info()) {
+      logger_->info(MSG_CONTENT_DISPOSITION_DETECTED,
+                    util::itos(cuid_).c_str(), contentDisposition.c_str());
     }
     return contentDisposition;
   }
@@ -116,158 +129,163 @@ std::string HttpResponse::determinFilename() const
 
 void HttpResponse::retrieveCookie()
 {
-  std::vector<std::string> v = httpHeader->get(HttpHeader::SET_COOKIE);
+  std::vector<std::string> v = httpHeader_->get(HttpHeader::SET_COOKIE);
   for(std::vector<std::string>::const_iterator itr = v.begin(), eoi = v.end();
       itr != eoi; ++itr) {
-    httpRequest->getCookieStorage()->parseAndStore(*itr,
-                                                   httpRequest->getHost(),
-                                                   httpRequest->getDir());
+    httpRequest_->getCookieStorage()->parseAndStore(*itr,
+                                                    httpRequest_->getHost(),
+                                                    httpRequest_->getDir());
   }
 }
 
 bool HttpResponse::isRedirect() const
 {
   const std::string& status = getResponseStatus();
-  return HttpHeader::S300 <= status && status < HttpHeader::S400 &&
-    httpHeader->defined(HttpHeader::LOCATION);
+  return (HttpHeader::S301 == status ||
+          HttpHeader::S302 == status ||
+          HttpHeader::S303 == status ||
+          HttpHeader::S307 == status) &&
+    httpHeader_->defined(HttpHeader::LOCATION);
 }
 
 void HttpResponse::processRedirect()
 {
   
-  if(httpRequest->getRequest()->redirectUri(getRedirectURI())) {
-    if(logger->info()) {
-      logger->info(MSG_REDIRECT, util::itos(cuid).c_str(),
-                   httpRequest->getRequest()->getCurrentUri().c_str());
+  if(httpRequest_->getRequest()->redirectUri(getRedirectURI())) {
+    if(logger_->info()) {
+      logger_->info(MSG_REDIRECT, util::itos(cuid_).c_str(),
+                    httpRequest_->getRequest()->getCurrentUri().c_str());
     }
   } else {
     throw DL_RETRY_EX
       (StringFormat("CUID#%s - Redirect to %s failed. It may not be a valid"
-                    " URI.", util::itos(cuid).c_str(),
-                    httpRequest->getRequest()->getCurrentUri().c_str()).str());
+                    " URI.", util::itos(cuid_).c_str(),
+                    httpRequest_->getRequest()->getCurrentUri().c_str()).str());
   }
 }
 
 std::string HttpResponse::getRedirectURI() const
 {
-  return httpHeader->getFirst(HttpHeader::LOCATION);
+  return httpHeader_->getFirst(HttpHeader::LOCATION);
 }
 
 bool HttpResponse::isTransferEncodingSpecified() const
 {
-  return httpHeader->defined(HttpHeader::TRANSFER_ENCODING);
+  return httpHeader_->defined(HttpHeader::TRANSFER_ENCODING);
 }
 
 std::string HttpResponse::getTransferEncoding() const
 {
-  // TODO See TODO in getTransferEncodingDecoder()
-  return httpHeader->getFirst(HttpHeader::TRANSFER_ENCODING);
+  // TODO See TODO in getTransferEncodingStreamFilter()
+  return httpHeader_->getFirst(HttpHeader::TRANSFER_ENCODING);
 }
 
-SharedHandle<Decoder> HttpResponse::getTransferEncodingDecoder() const
+SharedHandle<StreamFilter> HttpResponse::getTransferEncodingStreamFilter() const
 {
+  SharedHandle<StreamFilter> filter;
   // TODO Transfer-Encoding header field can contains multiple tokens. We should
   // parse the field and retrieve each token.
   if(isTransferEncodingSpecified()) {
     if(getTransferEncoding() == HttpHeader::CHUNKED) {
-      return SharedHandle<Decoder>(new ChunkedDecoder());
+      filter.reset(new ChunkedDecodingStreamFilter());
     }
   }
-  return SharedHandle<Decoder>();
+  return filter;
 }
 
 bool HttpResponse::isContentEncodingSpecified() const
 {
-  return httpHeader->defined(HttpHeader::CONTENT_ENCODING);
+  return httpHeader_->defined(HttpHeader::CONTENT_ENCODING);
 }
 
 const std::string& HttpResponse::getContentEncoding() const
 {
-  return httpHeader->getFirst(HttpHeader::CONTENT_ENCODING);
+  return httpHeader_->getFirst(HttpHeader::CONTENT_ENCODING);
 }
 
-SharedHandle<Decoder> HttpResponse::getContentEncodingDecoder() const
+SharedHandle<StreamFilter> HttpResponse::getContentEncodingStreamFilter() const
 {
+  SharedHandle<StreamFilter> filter;
 #ifdef HAVE_LIBZ
   if(getContentEncoding() == HttpHeader::GZIP ||
      getContentEncoding() == HttpHeader::DEFLATE) {
-    return SharedHandle<Decoder>(new GZipDecoder());
+    filter.reset(new GZipDecodingStreamFilter());
   }
 #endif // HAVE_LIBZ
-  return SharedHandle<Decoder>();
+  return filter;
 }
 
 uint64_t HttpResponse::getContentLength() const
 {
-  if(httpHeader.isNull()) {
+  if(httpHeader_.isNull()) {
     return 0;
   } else {
-    return httpHeader->getRange()->getContentLength();
+    return httpHeader_->getRange()->getContentLength();
   }
 }
 
 uint64_t HttpResponse::getEntityLength() const
 {
-  if(httpHeader.isNull()) {
+  if(httpHeader_.isNull()) {
     return 0;
   } else {
-    return httpHeader->getRange()->getEntityLength();
+    return httpHeader_->getRange()->getEntityLength();
   }
 }
 
 std::string HttpResponse::getContentType() const
 {
-  if(httpHeader.isNull()) {
+  if(httpHeader_.isNull()) {
     return A2STR::NIL;
   } else {
     return
-      util::split(httpHeader->getFirst(HttpHeader::CONTENT_TYPE), ";").first;
+      util::split(httpHeader_->getFirst(HttpHeader::CONTENT_TYPE), ";").first;
   }
 }
 
 void HttpResponse::setHttpHeader(const SharedHandle<HttpHeader>& httpHeader)
 {
-  this->httpHeader = httpHeader;
+  httpHeader_ = httpHeader;
 }
 
 void HttpResponse::setHttpRequest(const SharedHandle<HttpRequest>& httpRequest)
 {
-  this->httpRequest = httpRequest;
+  httpRequest_ = httpRequest;
 }
 
 // TODO return std::string
 const std::string& HttpResponse::getResponseStatus() const
 {
-  return httpHeader->getResponseStatus();
+  return httpHeader_->getResponseStatus();
 }
 
 bool HttpResponse::hasRetryAfter() const
 {
-  return httpHeader->defined(HttpHeader::RETRY_AFTER);
+  return httpHeader_->defined(HttpHeader::RETRY_AFTER);
 }
 
 time_t HttpResponse::getRetryAfter() const
 {
-  return httpHeader->getFirstAsUInt(HttpHeader::RETRY_AFTER);
+  return httpHeader_->getFirstAsUInt(HttpHeader::RETRY_AFTER);
 }
 
 Time HttpResponse::getLastModifiedTime() const
 {
-  return Time::parseHTTPDate(httpHeader->getFirst(HttpHeader::LAST_MODIFIED));
+  return Time::parseHTTPDate(httpHeader_->getFirst(HttpHeader::LAST_MODIFIED));
 }
 
 bool HttpResponse::supportsPersistentConnection() const
 {
   std::string connection =
-    util::toLower(httpHeader->getFirst(HttpHeader::CONNECTION));
-  std::string version = httpHeader->getVersion();
+    util::toLower(httpHeader_->getFirst(HttpHeader::CONNECTION));
+  std::string version = httpHeader_->getVersion();
 
   return
     connection.find(HttpHeader::CLOSE) == std::string::npos &&
     (version == HttpHeader::HTTP_1_1 ||
      connection.find("keep-alive") != std::string::npos) &&
-    (!httpRequest->isProxyRequestSet() ||
-     util::toLower(httpHeader->getFirst("Proxy-Connection")).find("keep-alive")
+    (!httpRequest_->isProxyRequestSet() ||
+     util::toLower(httpHeader_->getFirst("Proxy-Connection")).find("keep-alive")
      != std::string::npos);
 }
 
