@@ -69,14 +69,23 @@
 #include "ServerStatMan.h"
 #include "FileAllocationEntry.h"
 #include "CheckIntegrityEntry.h"
+#include "StreamFilter.h"
+#include "SinkStreamFilter.h"
+#include "ChunkedDecodingStreamFilter.h"
+#include "uri.h"
+#ifdef HAVE_LIBZ
+# include "GZipDecodingStreamFilter.h"
+#endif // HAVE_LIBZ
 
 namespace aria2 {
 
-static SharedHandle<Decoder> getTransferEncodingDecoder
-(const SharedHandle<HttpResponse>& httpResponse);
+static SharedHandle<StreamFilter> getTransferEncodingStreamFilter
+(const SharedHandle<HttpResponse>& httpResponse,
+ const SharedHandle<StreamFilter>& delegate = SharedHandle<StreamFilter>());
 
-static SharedHandle<Decoder> getContentEncodingDecoder
-(const SharedHandle<HttpResponse>& httpResponse);
+static SharedHandle<StreamFilter> getContentEncodingStreamFilter
+(const SharedHandle<HttpResponse>& httpResponse,
+ const SharedHandle<StreamFilter>& delegate = SharedHandle<StreamFilter>());
 
 HttpResponseCommand::HttpResponseCommand
 (cuid_t cuid,
@@ -125,6 +134,8 @@ bool HttpResponseCommand::executeInternal()
       getFileEntry()->setLength(totalLength);
       getRequestGroup()->initPieceStorage();
       getPieceStorage()->markAllPiecesDone();
+      // Just set checksum verification done.
+      getDownloadContext()->setChecksumVerified(true);
       getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
                           util::itos(getRequestGroup()->getGID()).c_str(),
                           getRequestGroup()->getFirstFilePath().c_str());
@@ -134,7 +145,7 @@ bool HttpResponseCommand::executeInternal()
     } else if(httpResponse->getResponseStatus() == HttpHeader::S200 ||
               httpResponse->getResponseStatus() == HttpHeader::S206) {
       // Remote file is newer than local file. We allow overwrite.
-      getOption()->put(PREF_ALLOW_OVERWRITE, V_TRUE);
+      getOption()->put(PREF_ALLOW_OVERWRITE, A2_V_TRUE);
     }
   }
   if(httpResponse->getResponseStatus() >= HttpHeader::S300 &&
@@ -145,9 +156,12 @@ bool HttpResponseCommand::executeInternal()
     return skipResponseBody(httpResponse);
   }
   if(getFileEntry()->isUniqueProtocol()) {
-    // TODO redirection should be considered here. We need to parse
+    // Redirection should be considered here. We need to parse
     // original URI to get hostname.
-    getFileEntry()->removeURIWhoseHostnameIs(getRequest()->getHost());
+    uri::UriStruct us;
+    if(uri::parse(us, getRequest()->getUri())) {
+      getFileEntry()->removeURIWhoseHostnameIs(us.host);
+    }
   }
   if(getPieceStorage().isNull()) {
     uint64_t totalLength = httpResponse->getEntityLength();
@@ -198,12 +212,16 @@ bool HttpResponseCommand::executeInternal()
       // anyway.
       getPieceStorage()->getDiskAdaptor()->truncate(0);
       getDownloadEngine()->addCommand
-        (createHttpDownloadCommand(httpResponse,
-                                   getTransferEncodingDecoder(httpResponse),
-                                   getContentEncodingDecoder(httpResponse)));
+        (createHttpDownloadCommand
+         (httpResponse,
+          getTransferEncodingStreamFilter
+          (httpResponse,
+           getContentEncodingStreamFilter(httpResponse))));
     } else {
-      getDownloadEngine()->addCommand(createHttpDownloadCommand
-                    (httpResponse, getTransferEncodingDecoder(httpResponse)));
+      getDownloadEngine()->addCommand
+        (createHttpDownloadCommand
+         (httpResponse,
+          getTransferEncodingStreamFilter(httpResponse)));
     }
     return true;
   }
@@ -247,18 +265,11 @@ bool HttpResponseCommand::handleDefaultEncoding
     return true;
   }
 
-  BtProgressInfoFileHandle infoFile
-    (new DefaultBtProgressInfoFile(getDownloadContext(),
-                                   getPieceStorage(),
-                                   getOption().get()));
-  if(!infoFile->exists() && getRequestGroup()->downloadFinishedByFileLength()) {
-    getPieceStorage()->markAllPiecesDone();
-    getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                        util::itos(getRequestGroup()->getGID()).c_str(),
-                        getRequestGroup()->getFirstFilePath().c_str());
+  SharedHandle<CheckIntegrityEntry> checkEntry =
+    getRequestGroup()->createCheckIntegrityEntry();
+  if(checkEntry.isNull()) {
     return true;
   }
-  getRequestGroup()->loadAndOpenFile(infoFile);
   File file(getRequestGroup()->getFirstFilePath());
   // We have to make sure that command that has Request object must
   // have segment after PieceStorage is initialized. See
@@ -275,15 +286,19 @@ bool HttpResponseCommand::handleDefaultEncoding
      !segment.isNull() && segment->getPositionToWrite() == 0 &&
      !getRequest()->isPipeliningEnabled()) {
     command = createHttpDownloadCommand
-      (httpResponse, getTransferEncodingDecoder(httpResponse));
+      (httpResponse,
+       getTransferEncodingStreamFilter(httpResponse));
   } else {
     getSegmentMan()->cancelSegment(getCuid());
     getFileEntry()->poolRequest(getRequest());
   }
   // After command is passed to prepareForNextAction(), it is managed
   // by CheckIntegrityEntry.
-  prepareForNextAction(command);
+  checkEntry->pushNextCommand(command);
   command = 0;
+
+  prepareForNextAction(checkEntry);
+
   if(getRequest()->getMethod() == Request::METHOD_HEAD) {
     poolConnection();
     getRequest()->setMethod(Request::METHOD_GET);
@@ -291,39 +306,49 @@ bool HttpResponseCommand::handleDefaultEncoding
   return true;
 }
 
-static SharedHandle<Decoder> getTransferEncodingDecoder
-(const SharedHandle<HttpResponse>& httpResponse)
+static SharedHandle<StreamFilter> getTransferEncodingStreamFilter
+(const SharedHandle<HttpResponse>& httpResponse,
+ const SharedHandle<StreamFilter>& delegate)
 {
-  SharedHandle<Decoder> decoder;
+  SharedHandle<StreamFilter> filter;
   if(httpResponse->isTransferEncodingSpecified()) {
-    decoder = httpResponse->getTransferEncodingDecoder();
-    if(decoder.isNull()) {
+    filter = httpResponse->getTransferEncodingStreamFilter();
+    if(filter.isNull()) {
       throw DL_ABORT_EX
         (StringFormat(EX_TRANSFER_ENCODING_NOT_SUPPORTED,
                       httpResponse->getTransferEncoding().c_str()).str());
     }
-    decoder->init();
+    filter->init();
+    filter->installDelegate(delegate);
   }
-  return decoder;
+  if(filter.isNull()) {
+    filter = delegate;
+  }
+  return filter;
 }
 
-static SharedHandle<Decoder> getContentEncodingDecoder
-(const SharedHandle<HttpResponse>& httpResponse)
+static SharedHandle<StreamFilter> getContentEncodingStreamFilter
+(const SharedHandle<HttpResponse>& httpResponse,
+ const SharedHandle<StreamFilter>& delegate)
 {
-  SharedHandle<Decoder> decoder;
+  SharedHandle<StreamFilter> filter;
   if(httpResponse->isContentEncodingSpecified()) {
-    decoder = httpResponse->getContentEncodingDecoder();
-    if(decoder.isNull()) {
+    filter = httpResponse->getContentEncodingStreamFilter();
+    if(filter.isNull()) {
       LogFactory::getInstance()->info
         ("Content-Encoding %s is specified, but the current implementation"
          "doesn't support it. The decoding process is skipped and the"
          "downloaded content will be still encoded.",
          httpResponse->getContentEncoding().c_str());
     } else {
-      decoder->init();
+      filter->init();
+      filter->installDelegate(delegate);
     }
   }
-  return decoder;
+  if(filter.isNull()) {
+    filter = delegate;
+  }
+  return filter;
 }
 
 bool HttpResponseCommand::handleOtherEncoding
@@ -343,10 +368,29 @@ bool HttpResponseCommand::handleOtherEncoding
     return prepareForRetry(0);
   }
 
+  // In this context, knowsTotalLength() is true only when the file is
+  // really zero-length.
+
+  SharedHandle<StreamFilter> streamFilter =
+    getTransferEncodingStreamFilter
+    (httpResponse,
+     getContentEncodingStreamFilter(httpResponse));
+  // If chunked transfer-encoding is specified, we have to read end of
+  // chunk markers(0\r\n\r\n, for example).
+  bool chunkedUsed = !streamFilter.isNull() &&
+    streamFilter->getName() == ChunkedDecodingStreamFilter::NAME;
+
   // For zero-length file, check existing file comparing its size
-  if(getRequestGroup()->downloadFinishedByFileLength()) {
+  if(!chunkedUsed && getDownloadContext()->knowsTotalLength() &&
+     getRequestGroup()->downloadFinishedByFileLength()) {
+    // TODO If metalink file does not contain size and it contains
+    // hash and file is not zero length, but remote server says the
+    // file size is 0, no hash check is performed in the current
+    // implementation. See also
+    // FtpNegotiationCommand::onFileSizeDetermined()
     getRequestGroup()->initPieceStorage();
     getPieceStorage()->markAllPiecesDone();
+    getDownloadContext()->setChecksumVerified(true);
     getLogger()->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
                         util::itos(getRequestGroup()->getGID()).c_str(),
                         getRequestGroup()->getFirstFilePath().c_str());
@@ -357,9 +401,12 @@ bool HttpResponseCommand::handleOtherEncoding
   getRequestGroup()->shouldCancelDownloadForSafety();
   getRequestGroup()->initPieceStorage();
   getPieceStorage()->getDiskAdaptor()->initAndOpenFile();
-  // In this context, knowsTotalLength() is true only when the file is
-  // really zero-length.
-  if(getDownloadContext()->knowsTotalLength()) {
+
+  // Local file size becomes zero when DiskAdaptor::initAndOpenFile()
+  // is called. So zero-length file is complete if chunked encoding is
+  // not used.
+  if(!chunkedUsed && getDownloadContext()->knowsTotalLength()) {
+    getRequestGroup()->getPieceStorage()->markAllPiecesDone();
     poolConnection();
     return true;
   }
@@ -369,16 +416,15 @@ bool HttpResponseCommand::handleOtherEncoding
   getSegmentMan()->getSegmentWithIndex(getCuid(), 0);
 
   getDownloadEngine()->addCommand
-    (createHttpDownloadCommand(httpResponse,
-                               getTransferEncodingDecoder(httpResponse),
-                               getContentEncodingDecoder(httpResponse)));
+    (createHttpDownloadCommand(httpResponse, streamFilter));
   return true;
 }
 
 bool HttpResponseCommand::skipResponseBody
 (const SharedHandle<HttpResponse>& httpResponse)
 {
-  SharedHandle<Decoder> decoder = getTransferEncodingDecoder(httpResponse);
+  SharedHandle<StreamFilter> filter =
+    getTransferEncodingStreamFilter(httpResponse);
   // We don't use Content-Encoding here because this response body is just
   // thrown away.
 
@@ -386,7 +432,7 @@ bool HttpResponseCommand::skipResponseBody
     (getCuid(), getRequest(), getFileEntry(), getRequestGroup(),
      httpConnection_, httpResponse,
      getDownloadEngine(), getSocket());
-  command->setTransferEncodingDecoder(decoder);
+  command->installStreamFilter(filter);
 
   // If request method is HEAD or the response body is zero-length,
   // set command's status to real time so that avoid read check blocking
@@ -403,10 +449,25 @@ bool HttpResponseCommand::skipResponseBody
   return true;
 }
 
+static bool decideFileAllocation
+(const SharedHandle<StreamFilter>& filter)
+{
+#ifdef HAVE_LIBZ
+  for(SharedHandle<StreamFilter> f = filter; !f.isNull(); f = f->getDelegate()){
+    // Since the compressed file's length are returned in the response header
+    // and the decompressed file size is unknown at this point, disable file
+    // allocation here.
+    if(f->getName() == GZipDecodingStreamFilter::NAME) {
+      return false;
+    }
+  }
+#endif // HAVE_LIBZ
+  return true;
+}
+
 HttpDownloadCommand* HttpResponseCommand::createHttpDownloadCommand
 (const SharedHandle<HttpResponse>& httpResponse,
- const SharedHandle<Decoder>& transferEncodingDecoder,
- const SharedHandle<Decoder>& contentEncodingDecoder)
+ const SharedHandle<StreamFilter>& filter)
 {
 
   HttpDownloadCommand* command =
@@ -417,16 +478,8 @@ HttpDownloadCommand* HttpResponseCommand::createHttpDownloadCommand
   command->setStartupIdleTime(getOption()->getAsInt(PREF_STARTUP_IDLE_TIME));
   command->setLowestDownloadSpeedLimit
     (getOption()->getAsInt(PREF_LOWEST_SPEED_LIMIT));
-  command->setTransferEncodingDecoder(transferEncodingDecoder);
-
-  if(!contentEncodingDecoder.isNull()) {
-    command->setContentEncodingDecoder(contentEncodingDecoder);
-    // Since the compressed file's length are returned in the response header
-    // and the decompressed file size is unknown at this point, disable file
-    // allocation here.
-    getRequestGroup()->setFileAllocationEnabled(false);
-  }
-
+  command->installStreamFilter(filter);
+  getRequestGroup()->setFileAllocationEnabled(decideFileAllocation(filter));    
   getRequestGroup()->getURISelector()->tuneDownloadCommand
     (getFileEntry()->getRemainingUris(), command);
 
@@ -444,6 +497,7 @@ void HttpResponseCommand::poolConnection()
 void HttpResponseCommand::onDryRunFileFound()
 {
   getPieceStorage()->markAllPiecesDone();
+  getDownloadContext()->setChecksumVerified(true);
   poolConnection();
 }
 
