@@ -41,6 +41,7 @@
 #include "prefs.h"
 #include "DlAbortEx.h"
 #include "BinaryStream.h"
+#include "MetalinkMetaurl.h"
 
 namespace aria2 {
 
@@ -49,7 +50,7 @@ MetalinkHelper::MetalinkHelper() {}
 MetalinkHelper::~MetalinkHelper() {}
 
 void MetalinkHelper::parseAndQuery
-(std::deque<SharedHandle<MetalinkEntry> >& result,
+(std::vector<SharedHandle<MetalinkEntry> >& result,
  const std::string& filename, const Option* option)
 {
   MetalinkProcessor proc;
@@ -58,7 +59,7 @@ void MetalinkHelper::parseAndQuery
 }
 
 void MetalinkHelper::parseAndQuery
-(std::deque<SharedHandle<MetalinkEntry> >& result,
+(std::vector<SharedHandle<MetalinkEntry> >& result,
  const SharedHandle<BinaryStream>& binaryStream, const Option* option)
 {
   MetalinkProcessor proc;
@@ -67,16 +68,50 @@ void MetalinkHelper::parseAndQuery
 }
 
 void MetalinkHelper::query
-(std::deque<SharedHandle<MetalinkEntry> >& result,
+(std::vector<SharedHandle<MetalinkEntry> >& result,
  const SharedHandle<Metalinker>& metalinker, const Option* option)
 {
-  if(metalinker->entries.empty()) {
-    throw DL_ABORT_EX("No file entry found. Probably, the metalink file is not configured properly or broken.");
-  }
   metalinker->queryEntry(result,
                          option->get(PREF_METALINK_VERSION),
                          option->get(PREF_METALINK_LANGUAGE),
                          option->get(PREF_METALINK_OS));
 }
 
+void MetalinkHelper::groupEntryByMetaurlName
+(std::vector<
+ std::pair<std::string, std::vector<SharedHandle<MetalinkEntry> > > >& result,
+ const std::vector<SharedHandle<MetalinkEntry> >& entries)
+{
+  for(std::vector<SharedHandle<MetalinkEntry> >::const_iterator eiter =
+        entries.begin(), eoi = entries.end(); eiter != eoi; ++eiter) {
+    if((*eiter)->metaurls.empty()) {
+      std::pair<std::string, std::vector<SharedHandle<MetalinkEntry> > > p;
+      p.second.push_back(*eiter);
+      result.push_back(p);
+    } else {
+      std::vector<
+      std::pair<std::string,
+        std::vector<SharedHandle<MetalinkEntry> > > >::iterator i =
+        result.begin();
+      if((*eiter)->metaurls[0]->name.empty() ||
+         !(*eiter)->sizeKnown) {
+        i = result.end();
+      }
+      for(; i != result.end(); ++i) {
+        if((*i).first == (*eiter)->metaurls[0]->url &&
+           !(*i).second[0]->metaurls[0]->name.empty()) {
+          (*i).second.push_back(*eiter);
+          break;
+        }
+      }
+      if(i == result.end()) {
+        std::pair<std::string, std::vector<SharedHandle<MetalinkEntry> > > p;
+        p.first = (*eiter)->metaurls[0]->url;
+        p.second.push_back(*eiter);
+        result.push_back(p);
+      }
+    }
+  }
+}
+
 } // namespace aria2
