@@ -94,6 +94,15 @@
 # include "BtPostDownloadHandler.h"
 # include "DHTSetup.h"
 # include "DHTRegistry.h"
+# include "DHTNode.h"
+# include "DHTRoutingTable.h"
+# include "DHTTaskQueue.h"
+# include "DHTTaskFactory.h"
+# include "DHTTokenTracker.h"
+# include "DHTMessageDispatcher.h"
+# include "DHTMessageReceiver.h"
+# include "DHTMessageFactory.h"
+# include "DHTMessageCallback.h"
 # include "BtMessageFactory.h"
 # include "BtRequestFactory.h"
 # include "BtMessageDispatcher.h"
@@ -155,6 +164,13 @@ RequestGroup::RequestGroup(const SharedHandle<Option>& option):
 
 RequestGroup::~RequestGroup() {}
 
+bool RequestGroup::isCheckIntegrityReady() const
+{
+  return option_->getAsBool(PREF_CHECK_INTEGRITY) &&
+    (downloadContext_->isChecksumVerificationAvailable() ||
+     downloadContext_->isPieceHashVerificationAvailable());
+}
+
 bool RequestGroup::downloadFinished() const
 {
   if(pieceStorage_.isNull()) {
@@ -198,6 +214,68 @@ void RequestGroup::closeFile()
   }
 }
 
+// TODO The function name is not intuitive at all.. it does not convey
+// that this function open file.
+SharedHandle<CheckIntegrityEntry> RequestGroup::createCheckIntegrityEntry()
+{
+  BtProgressInfoFileHandle infoFile
+    (new DefaultBtProgressInfoFile(downloadContext_, pieceStorage_,
+                                   option_.get()));
+  SharedHandle<CheckIntegrityEntry> checkEntry;
+  if(option_->getAsBool(PREF_CHECK_INTEGRITY) &&
+     downloadContext_->isPieceHashVerificationAvailable()) {
+    // When checking piece hash, we don't care file is downloaded and
+    // infoFile exists.
+    loadAndOpenFile(infoFile);
+    checkEntry.reset(new StreamCheckIntegrityEntry(this));
+  } else if(infoFile->exists()) {
+    loadAndOpenFile(infoFile);
+    if(downloadFinished()) {
+#ifdef ENABLE_MESSAGE_DIGEST
+      if(downloadContext_->isChecksumVerificationNeeded()) {
+        if(logger_->info()) {
+          logger_->info(MSG_HASH_CHECK_NOT_DONE);
+        }
+        SharedHandle<ChecksumCheckIntegrityEntry> tempEntry
+          (new ChecksumCheckIntegrityEntry(this));
+        tempEntry->setRedownload(true);
+        checkEntry = tempEntry;
+      } else
+#endif // ENABLE_MESSAGE_DIGEST
+        {
+          downloadContext_->setChecksumVerified(true);
+          logger_->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
+                          util::itos(gid_).c_str(),
+                          downloadContext_->getBasePath().c_str());
+        }
+    } else {
+      checkEntry.reset(new StreamCheckIntegrityEntry(this));
+    }
+  } else if(downloadFinishedByFileLength()) {
+    pieceStorage_->markAllPiecesDone();
+#ifdef ENABLE_MESSAGE_DIGEST
+    if(option_->getAsBool(PREF_CHECK_INTEGRITY) &&
+       downloadContext_->isChecksumVerificationAvailable()) {
+      loadAndOpenFile(infoFile);
+      SharedHandle<ChecksumCheckIntegrityEntry> tempEntry
+        (new ChecksumCheckIntegrityEntry(this));
+      tempEntry->setRedownload(true);
+      checkEntry = tempEntry;
+    } else
+#endif // ENABLE_MESSAGE_DIGEST
+      {
+        downloadContext_->setChecksumVerified(true);
+        logger_->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
+                        util::itos(gid_).c_str(),
+                        downloadContext_->getBasePath().c_str());
+      }
+  } else {
+    loadAndOpenFile(infoFile);
+    checkEntry.reset(new StreamCheckIntegrityEntry(this));
+  }
+  return checkEntry;
+}
+
 void RequestGroup::createInitialCommand
 (std::vector<Command*>& commands, DownloadEngine* e)
 {
@@ -283,10 +361,20 @@ void RequestGroup::createInitialCommand
                                 SharedHandle<BtProgressInfoFile>
                                 (progressInfoFile))));
       if(metadataGetMode) {
-        if(option_->getAsBool(PREF_ENABLE_DHT)) {
-          std::vector<Command*> dhtCommands;
-          DHTSetup().setup(dhtCommands, e);
-          e->addCommand(dhtCommands);
+        if(option_->getAsBool(PREF_ENABLE_DHT) ||
+           (!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+            option_->getAsBool(PREF_ENABLE_DHT6))) {
+          if(option_->getAsBool(PREF_ENABLE_DHT)) {
+            std::vector<Command*> dhtCommands;
+            DHTSetup().setup(dhtCommands, e, AF_INET);
+            e->addCommand(dhtCommands);
+          }
+          if(!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+             option_->getAsBool(PREF_ENABLE_DHT6)) {
+            std::vector<Command*> dhtCommands;
+            DHTSetup().setup(dhtCommands, e, AF_INET6);
+            e->addCommand(dhtCommands);
+          }
         } else {
           logger_->notice("For BitTorrent Magnet URI, enabling DHT is strongly"
                           " recommended. See --enable-dht option.");
@@ -343,16 +431,27 @@ void RequestGroup::createInitialCommand
       }
       progressInfoFile_ = progressInfoFile;
 
-      if(!torrentAttrs->privateTorrent && option_->getAsBool(PREF_ENABLE_DHT)) {
-        std::vector<Command*> dhtCommands;
-        DHTSetup().setup(dhtCommands, e);
-        e->addCommand(dhtCommands);
+      if(!torrentAttrs->privateTorrent &&
+         (option_->getAsBool(PREF_ENABLE_DHT) ||
+          (!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+           option_->getAsBool(PREF_ENABLE_DHT6)))) {
+        if(option_->getAsBool(PREF_ENABLE_DHT)) {
+          std::vector<Command*> dhtCommands;
+          DHTSetup().setup(dhtCommands, e, AF_INET);
+          e->addCommand(dhtCommands);
+        }
+        if(!e->getOption()->getAsBool(PREF_DISABLE_IPV6) &&
+           option_->getAsBool(PREF_ENABLE_DHT6)) {
+          std::vector<Command*> dhtCommands;
+          DHTSetup().setup(dhtCommands, e, AF_INET6);
+          e->addCommand(dhtCommands);
+        }
         const std::vector<std::pair<std::string, uint16_t> >& nodes =
           torrentAttrs->nodes;
-        if(!nodes.empty() && DHTSetup::initialized()) {
-          std::vector<std::pair<std::string, uint16_t> > entryPoints(nodes);
+        // TODO Are nodes in torrent IPv4 only?
+        if(!nodes.empty() && DHTRegistry::isInitialized()) {
           DHTEntryPointNameResolveCommand* command =
-            new DHTEntryPointNameResolveCommand(e->newCUID(), e, entryPoints);
+            new DHTEntryPointNameResolveCommand(e->newCUID(), e, nodes);
           command->setTaskQueue(DHTRegistry::getData().taskQueue);
           command->setTaskFactory(DHTRegistry::getData().taskFactory);
           command->setRoutingTable(DHTRegistry::getData().routingTable);
@@ -379,7 +478,7 @@ void RequestGroup::createInitialCommand
     if(option_->getAsBool(PREF_DRY_RUN) ||
        downloadContext_->getTotalLength() == 0) {
       createNextCommand(commands, e, 1);
-    }else {
+    } else {
       if(e->getRequestGroupMan()->isSameFileBeingDownloaded(this)) {
         throw DOWNLOAD_FAILURE_EXCEPTION
           (StringFormat(EX_DUPLICATE_FILE_DOWNLOAD,
@@ -391,38 +490,20 @@ void RequestGroup::createInitialCommand
                                            SharedHandle<PieceStorage>(),
                                            option_.get())));
       initPieceStorage();
-      BtProgressInfoFileHandle infoFile
-        (new DefaultBtProgressInfoFile(downloadContext_, pieceStorage_,
-                                       option_.get()));
-      if(!infoFile->exists() && downloadFinishedByFileLength()) {
-        pieceStorage_->markAllPiecesDone();
-        logger_->notice(MSG_DOWNLOAD_ALREADY_COMPLETED,
-                        util::itos(gid_).c_str(),
-                        downloadContext_->getBasePath().c_str());
-      } else {
-        loadAndOpenFile(infoFile);
-#ifdef ENABLE_MESSAGE_DIGEST
-        if(downloadFinished() &&
-           downloadContext_->isChecksumVerificationNeeded()) {
-          if(logger_->info()) {
-            logger_->info(MSG_HASH_CHECK_NOT_DONE);
-          }
-          SharedHandle<CheckIntegrityEntry> entry
-            (new ChecksumCheckIntegrityEntry(this));
-          if(entry->isValidationReady()) {
-            entry->initValidator();
-            entry->cutTrailingGarbage();
-            e->getCheckIntegrityMan()->pushEntry(entry);
-            return;
-          }
-        }
-#endif // ENABLE_MESSAGE_DIGEST
-        SharedHandle<CheckIntegrityEntry> checkIntegrityEntry
-          (new StreamCheckIntegrityEntry(this));
-        processCheckIntegrityEntry(commands, checkIntegrityEntry, e);
+      SharedHandle<CheckIntegrityEntry> checkEntry =
+        createCheckIntegrityEntry();
+      if(!checkEntry.isNull()) {
+        processCheckIntegrityEntry(commands, checkEntry, e);
       }
     }
   } else {
+    // TODO --dry-run is not supported for multifile download for now.
+    if(option_->getAsBool(PREF_DRY_RUN)) {
+      throw DOWNLOAD_FAILURE_EXCEPTION
+        ("--dry-run in multi-file download is not supported yet.");
+    }
+    // TODO file size is known in this context?
+
     // In this context, multiple FileEntry objects are in
     // DownloadContext.
     if(e->getRequestGroupMan()->isSameFileBeingDownloaded(this)) {
@@ -446,7 +527,7 @@ void RequestGroup::createInitialCommand
       pieceStorage_->getDiskAdaptor()->openFile();
     } else {
       if(pieceStorage_->getDiskAdaptor()->fileExists()) {
-        if(!option_->getAsBool(PREF_CHECK_INTEGRITY) &&
+        if(!isCheckIntegrityReady() &&
            !option_->getAsBool(PREF_ALLOW_OVERWRITE)) {
           // TODO we need this->haltRequested = true?
           throw DOWNLOAD_FAILURE_EXCEPTION
@@ -472,11 +553,15 @@ void RequestGroup::processCheckIntegrityEntry
  const SharedHandle<CheckIntegrityEntry>& entry,
  DownloadEngine* e)
 {
+  uint64_t actualFileSize = pieceStorage_->getDiskAdaptor()->size();
+  if(actualFileSize > downloadContext_->getTotalLength()) {
+    entry->cutTrailingGarbage();
+  }
 #ifdef ENABLE_MESSAGE_DIGEST
-  if(option_->getAsBool(PREF_CHECK_INTEGRITY) &&
+  if((option_->getAsBool(PREF_CHECK_INTEGRITY) ||
+      downloadContext_->isChecksumVerificationNeeded()) &&
      entry->isValidationReady()) {
     entry->initValidator();
-    entry->cutTrailingGarbage();
     // Don't save control file(.aria2 file) when user presses
     // control-c key while aria2 is checking hashes. If control file
     // doesn't exist when aria2 launched, the completed length in
@@ -495,7 +580,14 @@ void RequestGroup::processCheckIntegrityEntry
 void RequestGroup::initPieceStorage()
 {
   SharedHandle<PieceStorage> tempPieceStorage;
-  if(downloadContext_->knowsTotalLength()) {
+  if(downloadContext_->knowsTotalLength() &&
+     // Following conditions are needed for chunked encoding with
+     // content-length = 0. Google's dl server used this before.
+     (downloadContext_->getTotalLength() > 0
+#ifdef ENABLE_BITTORRENT
+      || downloadContext_->hasAttribute(bittorrent::BITTORRENT)
+#endif // ENABLE_BITTORRENT
+      )) {
 #ifdef ENABLE_BITTORRENT
     SharedHandle<DefaultPieceStorage> ps
       (new DefaultPieceStorage(downloadContext_, option_.get()));
@@ -562,9 +654,7 @@ bool RequestGroup::downloadFinishedByFileLength()
 {
   // assuming that a control file doesn't exist.
   if(!isPreLocalFileCheckEnabled() ||
-     option_->getAsBool(PREF_ALLOW_OVERWRITE) ||
-     (option_->getAsBool(PREF_CHECK_INTEGRITY) &&
-      !downloadContext_->getPieceHashes().empty())) {
+     option_->getAsBool(PREF_ALLOW_OVERWRITE)) {
     return false;
   }
   if(!downloadContext_->knowsTotalLength()) {
@@ -603,7 +693,7 @@ void RequestGroup::adjustFilename
       // File exists but user decided to resume it.
     } else {
 #ifdef ENABLE_MESSAGE_DIGEST
-      if(outfile.exists() && option_->getAsBool(PREF_CHECK_INTEGRITY)) {
+      if(outfile.exists() && isCheckIntegrityReady()) {
         // check-integrity existing file
       } else {
 #endif // ENABLE_MESSAGE_DIGEST
@@ -647,7 +737,7 @@ void RequestGroup::loadAndOpenFile(const BtProgressInfoFileHandle& progressInfoF
         pieceStorage_->markPiecesDone(outfile.size());
       } else {
 #ifdef ENABLE_MESSAGE_DIGEST
-        if(outfile.exists() && option_->getAsBool(PREF_CHECK_INTEGRITY)) {
+        if(outfile.exists() && isCheckIntegrityReady()) {
           pieceStorage_->getDiskAdaptor()->openExistingFile();
         } else {
 #endif // ENABLE_MESSAGE_DIGEST
