@@ -71,6 +71,7 @@
 #include "CheckIntegrityEntry.h"
 #include "Segment.h"
 #include "DlAbortEx.h"
+#include "uri.h"
 
 namespace aria2 {
 
@@ -87,7 +88,10 @@ RequestGroupMan::RequestGroupMan
   (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)),
   maxOverallUploadSpeedLimit_(option->getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT)),
   xmlRpc_(option->getAsBool(PREF_ENABLE_XML_RPC)),
-  queueCheck_(true)
+  queueCheck_(true),
+  removedErrorResult_(0),
+  removedLastErrorResult_(downloadresultcode::FINISHED),
+  maxDownloadResult_(option->getAsInt(PREF_MAX_DOWNLOAD_RESULT))
 {}
 
 bool RequestGroupMan::downloadFinished()
@@ -267,6 +271,7 @@ static void executeStopHook
   }
 }
 
+namespace {
 class ProcessStoppedRequestGroup {
 private:
   DownloadEngine* e_;
@@ -336,6 +341,9 @@ public:
             e_->getRequestGroupMan()->insertReservedGroup(0, nextGroups);
           }
         } else {
+          logger_->notice("Download GID#%s not complete: %s",
+                          util::itos(group->getGID()).c_str(),
+                          group->getDownloadContext()->getBasePath().c_str());
           group->saveControlFile();
         }
       } catch(RecoverableException& ex) {
@@ -343,23 +351,24 @@ public:
       }
       if(group->isPauseRequested()) {
         reservedGroups_.push_front(group);
-      } else {
-        downloadResults_.push_back(group->createDownloadResult());
-      }
-      group->releaseRuntimeResource(e_);
-      if(group->isPauseRequested()) {
+        group->releaseRuntimeResource(e_);
         group->setForceHaltRequested(false);
         util::executeHookByOptName
           (group, e_->getOption(), PREF_ON_DOWNLOAD_PAUSE);
         // TODO Should we have to prepend spend uris to remaining uris
         // in case PREF_REUSE_URI is disabed?
       } else {
-        executeStopHook(downloadResults_.back(), e_->getOption());
+        SharedHandle<DownloadResult> dr = group->createDownloadResult();
+        e_->getRequestGroupMan()->addDownloadResult(dr);
+        group->releaseRuntimeResource(e_);
+        executeStopHook(dr, e_->getOption());
       }
     }
   }
 };
+}
 
+namespace {
 class CollectServerStat {
 private:
   RequestGroupMan* requestGroupMan_;
@@ -401,13 +410,16 @@ public:
     }    
   }
 };
+}
 
+namespace {
 class FindStoppedRequestGroup {
 public:
   bool operator()(const SharedHandle<RequestGroup>& group) {
     return group->getNumCommand() == 0;
   }
 };
+}
 
 void RequestGroupMan::updateServerStat()
 {
@@ -509,7 +521,7 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
         logger_->debug("Commands deleted");
       }
       groupToAdd->releaseRuntimeResource(e);
-      downloadResults_.push_back(groupToAdd->createDownloadResult());
+      addDownloadResult(groupToAdd->createDownloadResult());
     }
   }
   if(!temp.empty()) {
@@ -551,9 +563,9 @@ void RequestGroupMan::closeFile()
 RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
 {
   size_t finished = 0;
-  size_t error = 0;
+  size_t error = removedErrorResult_;
   size_t inprogress = 0;
-  downloadresultcode::RESULT lastError = downloadresultcode::FINISHED;
+  downloadresultcode::RESULT lastError = removedLastErrorResult_;
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
         downloadResults_.begin(), eoi = downloadResults_.end();
       itr != eoi; ++itr) {
@@ -760,6 +772,42 @@ RequestGroupMan::findDownloadResult(gid_t gid) const
   return SharedHandle<DownloadResult>();
 }
 
+void RequestGroupMan::addDownloadResult(const SharedHandle<DownloadResult>& dr)
+{
+  if(maxDownloadResult_ == 0) {
+    if(!downloadResults_.empty()) {
+      for(std::deque<SharedHandle<DownloadResult> >::iterator i =
+            downloadResults_.begin(), eoi = downloadResults_.end(); i != eoi;
+          ++i) {
+        if((*i)->result != downloadresultcode::FINISHED) {
+          removedLastErrorResult_ = (*i)->result;
+          ++removedErrorResult_;
+        }
+      }
+      downloadResults_.clear();
+    }
+    if(dr->result != downloadresultcode::FINISHED) {
+      removedLastErrorResult_ = dr->result;
+      ++removedErrorResult_;
+    }
+  } else {
+    size_t curSize = downloadResults_.size();
+    if(curSize >= maxDownloadResult_) {
+      std::deque<SharedHandle<DownloadResult> >::iterator last =
+        downloadResults_.begin()+curSize-maxDownloadResult_+1;
+      for(std::deque<SharedHandle<DownloadResult> >::iterator i =
+            downloadResults_.begin(); i != last; ++i) {
+        if((*i)->result != downloadresultcode::FINISHED) {
+          removedLastErrorResult_ = (*i)->result;
+          ++removedErrorResult_;
+        }
+      }        
+      downloadResults_.erase(downloadResults_.begin(), last);
+    }
+    downloadResults_.push_back(dr);
+  }
+}
+
 void RequestGroupMan::purgeDownloadResult()
 {
   downloadResults_.clear();
@@ -850,25 +898,25 @@ bool RequestGroupMan::doesOverallUploadSpeedExceed()
 void RequestGroupMan::getUsedHosts
 (std::vector<std::pair<size_t, std::string> >& usedHosts)
 {
-  Request r;
   for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
         requestGroups_.begin(), eoi = requestGroups_.end(); i != eoi; ++i) {
     const std::deque<SharedHandle<Request> >& inFlightReqs =
       (*i)->getDownloadContext()->getFirstFileEntry()->getInFlightRequests();
     for(std::deque<SharedHandle<Request> >::const_iterator j =
           inFlightReqs.begin(), eoj = inFlightReqs.end(); j != eoj; ++j) {
-      if(r.setUri((*j)->getUri())) {
+      uri::UriStruct us;
+      if(uri::parse(us, (*j)->getUri())) {
         std::vector<std::pair<size_t, std::string> >::iterator k;
         std::vector<std::pair<size_t, std::string> >::iterator eok =
           usedHosts.end();
         for(k =  usedHosts.begin(); k != eok; ++k) {
-          if((*k).second == r.getHost()) {
+          if((*k).second == us.host) {
             ++(*k).first;
             break;
           }
         }
         if(k == eok) {
-          usedHosts.push_back(std::make_pair(1, r.getHost()));
+          usedHosts.push_back(std::make_pair(1, us.host));
         }
       }
     }
