@@ -38,7 +38,7 @@
 #include "common.h"
 
 #include <string>
-#include <deque>
+#include <vector>
 #include <set>
 
 #include "SharedHandle.h"
@@ -47,19 +47,24 @@ namespace aria2 {
 
 class RequestGroup;
 class Option;
+class MetadataInfo;
+class DownloadContext;
 
 const std::set<std::string>& listRequestOptions();
 
 #ifdef ENABLE_BITTORRENT
 // Create RequestGroup object using torrent file specified by
 // torrent-file option.  If non-empty torrentData is specified, then
-// it is used as a content of torrent file instead. In this function,
+// it is used as a content of torrent file instead. If
+// adjustAnnounceUri is true, announce URIs are adjusted using
+// bittorrent::adjustAnnounceUri().  In this function,
 // force-sequential is ignored.
 void createRequestGroupForBitTorrent
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
- const std::deque<std::string>& uris,
- const std::string& torrentData = "");
+ const std::vector<std::string>& uris,
+ const std::string& torrentData = "",
+ bool adjustAnnounceUri = true);
 #endif // ENABLE_BITTORRENT
 
 #ifdef ENABLE_METALINK
@@ -67,7 +72,7 @@ void createRequestGroupForBitTorrent
 // metalink-file option. If non-empty metalinkData is specified, it is
 // used as a content of metalink file instead.
 void createRequestGroupForMetalink
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
  const std::string& metalinkData = "");
 #endif // ENABLE_METALINK
@@ -78,18 +83,30 @@ void createRequestGroupForMetalink
 // The additional out and dir options can be specified after each line of URIs.
 // This optional line must start with white space(s).
 void createRequestGroupForUriList
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option);
 
 // Create RequestGroup object using provided uris.  If ignoreLocalPath
 // is true, a path to torrent file abd metalink file are ignored.
 void createRequestGroupForUri
-(std::deque<SharedHandle<RequestGroup> >& result,
+(std::vector<SharedHandle<RequestGroup> >& result,
  const SharedHandle<Option>& option,
- const std::deque<std::string>& uris,
+ const std::vector<std::string>& uris,
  bool ignoreForceSequential = false,
  bool ignoreLocalPath = false);
 
+template<typename InputIterator>
+void setMetadataInfo
+(InputIterator first, InputIterator last, const SharedHandle<MetadataInfo>& mi)
+{
+  for(; first != last; ++first) {
+    (*first)->setMetadataInfo(mi);
+  }
+}
+
+SharedHandle<MetadataInfo>
+createMetadataInfoFromFirstFileEntry(const SharedHandle<DownloadContext>& dctx);
+
 } // namespace aria2
 
 #endif // _D_DOWNLOAD_HELPER_H_
