@@ -687,10 +687,10 @@
     "                              added by aria2.addTorrent and aria2.addMetalink\n" \
     "                              XML-RPC method are not saved.")
 #define TEXT_MAX_CONNECTION_PER_SERVER          \
-  _(" --max-connection-per-server=NUM The maximum number of connections to one server\n"\
-    "                              for each download.")
+  _(" -x, --max-connection-per-server=NUM The maximum number of connections to one\n" \
+    "                              server for each download.")
 #define TEXT_MIN_SPLIT_SIZE                     \
-  _(" --min-split-size=SIZE        aria2 does not split less than 2*SIZE byte range.\n" \
+  _(" -k, --min-split-size=SIZE    aria2 does not split less than 2*SIZE byte range.\n" \
     "                              For example, let's consider downloading 20MiB\n" \
     "                              file. If SIZE is 10M, aria2 can split file into 2\n" \
     "                              range [0-10MiB) and [10MiB-20MiB) and download it\n" \
@@ -741,3 +741,15 @@
     "                              announce URIs. When specifying '*' in shell\n" \
     "                              command-line, don't forget to escape or quote it.\n" \
     "                              See also --bt-tracker option.")
+#define TEXT_MAX_DOWNLOAD_RESULT                \
+  _(" --max-download-result=NUM    Set maximum number of download result kept in\n" \
+    "                              memory. The download results are completed/error/\n" \
+    "                              removed downloads. The download results are stored\n" \
+    "                              in FIFO queue and it can store at most NUM\n" \
+    "                              download results. When queue is full and new\n" \
+    "                              download result is created, oldest download result\n" \
+    "                              is removed from the front of the queue and new one\n" \
+    "                              is pushed to the back. Setting big number in this\n" \
+    "                              option may result high memory consumption after\n" \
+    "                              thousands of downloads. Specifying 0 means no\n" \
+    "                              download result is kept.")
