@@ -1,4 +1,5 @@
 #include "File.h"
+#include "TestUtil.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -6,7 +7,7 @@
 #include <fstream>
 #include <cppunit/extensions/HelperMacros.h>
 
-using namespace std;
+namespace aria2 {
 
 class FileTest:public CppUnit::TestFixture {
 
@@ -20,6 +21,7 @@ class FileTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testGetDirname);
   CPPUNIT_TEST(testGetBasename);
   CPPUNIT_TEST(testRenameTo);
+  CPPUNIT_TEST(testUtime);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -36,6 +38,7 @@ public:
   void testGetDirname();
   void testGetBasename();
   void testRenameTo();
+  void testUtime();
 };
 
 
@@ -76,7 +79,7 @@ void FileTest::testIsDir() {
 
 void FileTest::testRemove() {
   int fd;
-  string name = "/tmp/aria2test";
+  std::string name = "./aria2_FileTest_testRemove_testregfile";
   unlink(name.c_str());
   if((fd = creat(name.c_str(), S_IRUSR|S_IWUSR)) < 0) {
     CPPUNIT_FAIL("cannot create test file");
@@ -89,7 +92,7 @@ void FileTest::testRemove() {
   // delete the file again
   CPPUNIT_ASSERT(!f.remove());
 
-  string dir = "/tmp/aria2testdir";
+  std::string dir = "./aria2_FileTest_testRemove_testdir";
 #ifdef __MINGW32__
   mkdir(dir.c_str());
 #else
@@ -109,7 +112,7 @@ void FileTest::testSize() {
 }
 
 void FileTest::testMkdir() {
-  string dir = "/tmp/aria2test2/test";
+  std::string dir = "./aria2_FileTest_testMkdir/test";
   File d(dir);
   if(d.exists()) {
     CPPUNIT_ASSERT(d.remove());
@@ -126,24 +129,76 @@ void FileTest::testMkdir() {
 
 void FileTest::testGetDirname()
 {
-  File f("/tmp/dist/aria2.tar.bz2");
-  CPPUNIT_ASSERT_EQUAL(string("/tmp/dist"), f.getDirname());
+  {
+    File f("/usr/lib");
+    CPPUNIT_ASSERT_EQUAL(std::string("/usr"), f.getDirname());
+  }
+  {
+    File f("/usr/");
+    CPPUNIT_ASSERT_EQUAL(std::string("/usr"), f.getDirname());
+  }
+  {
+    File f("usr");
+    CPPUNIT_ASSERT_EQUAL(std::string("."), f.getDirname());
+  }
+  {
+    File f("/");
+    CPPUNIT_ASSERT_EQUAL(std::string("/"), f.getDirname());
+  }
+  {
+    File f(".");
+    CPPUNIT_ASSERT_EQUAL(std::string("."), f.getDirname());
+  }
+  {
+    File f("..");
+    CPPUNIT_ASSERT_EQUAL(std::string("."), f.getDirname());
+  }
+  {
+    File f("");
+    CPPUNIT_ASSERT_EQUAL(std::string(""), f.getDirname());
+  }  
 }
 
 void FileTest::testGetBasename()
 {
-  File f("/tmp/dist/aria2.tar.bz2");
-  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2"), f.getBasename());
+  {
+    File f("/usr/lib");
+    CPPUNIT_ASSERT_EQUAL(std::string("lib"), f.getBasename());
+  }
+  {
+    File f("/usr/");
+    CPPUNIT_ASSERT_EQUAL(std::string(""), f.getBasename());
+  }
+  {
+    File f("usr");
+    CPPUNIT_ASSERT_EQUAL(std::string("usr"), f.getBasename());
+  }
+  {
+    File f("/");
+    CPPUNIT_ASSERT_EQUAL(std::string(""), f.getBasename());
+  }
+  {
+    File f(".");
+    CPPUNIT_ASSERT_EQUAL(std::string("."), f.getBasename());
+  }
+  {
+    File f("..");
+    CPPUNIT_ASSERT_EQUAL(std::string(".."), f.getBasename());
+  }
+  {
+    File f("");
+    CPPUNIT_ASSERT_EQUAL(std::string(""), f.getBasename());
+  }
 }
 
 void FileTest::testRenameTo()
 {
-  string fname = "FileTest_testRenameTo.txt";
-  ofstream of(fname.c_str());
+  std::string fname = "aria2_FileTest_testRenameTo.txt";
+  std::ofstream of(fname.c_str(), std::ios::binary);
   of.close();
 
   File f(fname);
-  string fnameTo = "FileTest_testRenameTo_dest.txt";
+  std::string fnameTo = "aria2_FileTest_testRenameTo_dest.txt";
   CPPUNIT_ASSERT(f.renameTo(fnameTo));
   CPPUNIT_ASSERT(f.exists());
   CPPUNIT_ASSERT(!File(fname).exists());
@@ -155,3 +210,25 @@ void FileTest::testRenameTo()
   
   CPPUNIT_ASSERT(f.renameTo(fname));
 }
+
+void FileTest::testUtime()
+{
+  File f("./aria2_FileTest_testUTime");
+  createFile(f.getPath(), 0);
+  
+  time_t atime = (time_t) 100000;
+  time_t mtime = (time_t) 200000;
+  
+  CPPUNIT_ASSERT(f.utime(Time(atime), Time(mtime)));
+  
+  a2_struct_stat buf;
+  CPPUNIT_ASSERT(0 == a2stat(f.getPath().c_str(), &buf));
+  CPPUNIT_ASSERT_EQUAL((time_t)atime, buf.st_atime);
+  CPPUNIT_ASSERT_EQUAL((time_t)mtime, f.getModifiedTime().getTime());
+
+  File notFound("./aria2_FileTest_testUTime_notFound");
+  notFound.remove();
+  CPPUNIT_ASSERT(!notFound.utime(Time(atime), Time(mtime)));
+}
+
+} // namespace aria2
